package de.xoev.transport.xta;

import org.jetbrains.annotations.Nullable;

import javax.net.ssl.SSLEngine;
import javax.net.ssl.X509ExtendedKeyManager;
import java.net.Socket;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Enumeration;

class SimpleKeyManager extends X509ExtendedKeyManager {

    private final KeyStore myPkStore;
    private final String myKeyAlias;
    private final String myPassword;

    public SimpleKeyManager(KeyStore pkStore, @Nullable String keyAlias, String password) {
        myPkStore = pkStore;
        myKeyAlias = keyAlias != null ? keyAlias : firstKeyAlias(pkStore);
        myPassword = password;
    }

    private static String firstKeyAlias(KeyStore pkStore) {
        try {
            final Enumeration<String> aliases = pkStore.aliases();
            while (aliases.hasMoreElements()) {
                final String alias = aliases.nextElement();
                if (pkStore.isKeyEntry(alias)) {
                    return alias;
                }
            }
            throw new IllegalStateException("No key entry found");
        } catch (KeyStoreException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public String[] getClientAliases(String s, Principal[] principals) {
        return new String[]{ myKeyAlias };
    }

    @Override
    public String chooseEngineClientAlias(String[] strings, Principal[] principals, SSLEngine sslEngine) {
        return chooseClientAlias(strings, principals, null);
    }

    @Override
    public String chooseClientAlias(String[] strings, Principal[] principals, Socket socket) {
        return myKeyAlias;
    }

    @Override
    public String[] getServerAliases(String s, Principal[] principals) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String chooseServerAlias(String s, Principal[] principals, Socket socket) {
        throw new UnsupportedOperationException();
    }

    @Override
    public X509Certificate[] getCertificateChain(String s) {
        try {
            return Arrays.asList(myPkStore.getCertificateChain(s)).toArray(new X509Certificate[0]);
        } catch (KeyStoreException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public PrivateKey getPrivateKey(String s) {
        try {
            return (PrivateKey) myPkStore.getKey(s, myPassword.toCharArray());
        } catch (KeyStoreException | NoSuchAlgorithmException | UnrecoverableKeyException e) {
            throw new RuntimeException(e);
        }
    }
}