package de.berlin.bda.xta2.receiver;

import de.xoev.transport.xta.XTAServiceManager;
import de.xoev.transport.xta._211.*;
import eu.osci.ws._2008._05.transport.*;
import eu.osci.ws._2014._10.transport.MessageMetaData;
import eu.osci.ws._2014._10.transport.PartyIdentifierType;
import eu.osci.ws._2014._10.transport.PartyType;
import org.w3._2005._08.addressing.AttributedURIType;

import javax.xml.ws.Holder;
import java.math.BigInteger;
import java.util.List;

public class Xta2Receiver {

    public static void main(String[] args) throws Exception {

        // ServiceManager fuer SSL-Kommunikation initialisieren
        final XTAServiceManager manager = new XTAServiceManager(
                "MY_KEYSTORE.p12", null, "MY_KEYSTORE_PW",
                "MY_TRUSTSTORE.jks", "MY_TRUSTSTORE_PW");

        // Basisadresse des GCG der DEV-Umgebung
        final String xtaBase = "https://cc01lsv2771.srz.lit.verwalt-berlin.de:8444/GovGateway211";

        final ManagementPortType mgmtPort = manager.getMgmtPort(xtaBase + "/management211");
        final MsgBoxPortType msgPort = manager.getFetchPort(xtaBase + "/fetch211");
        final SendPortType sendPort = manager.getSendPort(xtaBase + "/send211");

        final String name = "USER";

        final PartyType authorIdentifier = createIdentifier(name);

        // Nachricht(en) abholen
        fetchMessages(msgPort, authorIdentifier);
    }


    static void fetchMessages(MsgBoxPortType msgPort, PartyType authorIdentifier) throws Exception {

        // StatusRequest - auf max. 10 ergebnisse beschraenkt
        final MsgBoxStatusListRequestType statusRequest = new MsgBoxStatusListRequestType();
        statusRequest.setMaxListItems(BigInteger.valueOf(10));

        // Liste verfuegbarer Nachrichten abholen
        final Holder<MsgBoxResponseType> fetchResponseHeader = new Holder<>();
        final MsgStatusListType statusList = msgPort.getStatusList(statusRequest, authorIdentifier, fetchResponseHeader);
        final List<MessageMetaData> messageMetaDataList = statusList.getMessageMetaData();

        final MsgBoxResponseType fetchResponse = fetchResponseHeader.value;
        final String msgBoxRequestID = fetchResponse.getMsgBoxRequestID();
        final BigInteger itemsPending = fetchResponse.getItemsPending();
        System.out.println("msgBoxRequestID = " + msgBoxRequestID + ", pending: " + itemsPending);

        // CloseRequest quittiert alle erhaltenen Nachrichten
        final MsgBoxCloseRequestType closeRequest = new MsgBoxCloseRequestType();
        closeRequest.setMsgBoxRequestID(msgBoxRequestID);

        for (MessageMetaData metaData : messageMetaDataList) {
            final AttributedURIType messageID = metaData.getMsgIdentification().getMessageID();
            System.out.println("messageID = " + messageID.getValue());

            // FetchRequest fuer naechste MessageID aus StatusRequest
            final MsgBoxFetchRequest fetchRequest = new MsgBoxFetchRequest();
            final MsgSelector msgSelector = new MsgSelector();
            msgSelector.getMessageID().add(messageID);
            fetchRequest.setMsgSelector(msgSelector);

            // Nachricht abholen...
            final Holder<MessageMetaData> messageMetaDataHolder = new Holder<>();
            final Holder<MsgBoxResponseType> responseHeaderHolder = new Holder<>();
            final GenericContentContainer container = msgPort.getMessage(fetchRequest, authorIdentifier, messageMetaDataHolder, responseHeaderHolder);

            // muss die gleiche ID wie aus dem StatusRequest sein
            final MessageMetaData messageMetaData = messageMetaDataHolder.value;
            assert messageMetaData.getMsgIdentification().getMessageID().equals(messageID);

            // .. und Nachricht und Metadaten verarbeiten
            final ContentType message = container.getContentContainer().getMessage();
            final String subject = messageMetaData.getQualifier().getSubject();
            System.out.println("subject = " + subject);

            final String service = messageMetaData.getQualifier().getService();
            System.out.println("service = " + service);

            // eigentlicher Nachrichteninhalt
            final String value = new String(message.getValue(), message.getEncoding());
            System.out.println("value = " + value);

            // empfangene messageid zu CloseRequest hinzufuegen
            closeRequest.getLastMsgReceived().add(messageID);
        }

        // alle Nachrichten quittieren
        msgPort.close(closeRequest, authorIdentifier);
    }

    static PartyType createIdentifier(final String name) {
        final PartyType authorIdentifier = new PartyType();
        final PartyIdentifierType identifier = new PartyIdentifierType();

        // abhaengig von GCG-Konfiguration
        identifier.setType("UserAdministration");
        identifier.setValue(name);

        authorIdentifier.setIdentifier(identifier);
        return authorIdentifier;
    }
}
